/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.kappmenu.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef APPMENU_INTERFACE_H
#define APPMENU_INTERFACE_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.kde.kappmenu
 */
class OrgKdeKappmenuInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.kde.kappmenu"; }

public:
    OrgKdeKappmenuInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgKdeKappmenuInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> reconfigure()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("reconfigure"), argumentList);
    }

    inline QDBusPendingReply<> showMenu(int x, int y, const QString &service, const QDBusObjectPath &objectPath, int actionId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(x) << QVariant::fromValue(y) << QVariant::fromValue(service) << QVariant::fromValue(objectPath) << QVariant::fromValue(actionId);
        return asyncCallWithArgumentList(QStringLiteral("showMenu"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void menuHidden(const QString &service, const QDBusObjectPath &objectPath);
    void menuShown(const QString &service, const QDBusObjectPath &objectPath);
    void reconfigured();
    void showRequest(const QString &service, const QDBusObjectPath &objectPath, int actionId);
};

namespace org {
  namespace kde {
    using kappmenu = ::OrgKdeKappmenuInterface;
  }
}

#endif
