// This file is generated by kconfig_compiler_kf6 from slidingpopups.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_SLIDINGPOPUPSCONFIG_H
#define KWIN_SLIDINGPOPUPSCONFIG_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {

class SlidingPopupsConfig : public KConfigSkeleton
{
  public:

    static SlidingPopupsConfig *self();
    static void instance(const QString& cfgfilename);
    static void instance(KSharedConfig::Ptr config);
    ~SlidingPopupsConfig() override;

    /**
      Set SlideInTime
    */
    static
    void setSlideInTime( int v )
    {
      if (!self()->isSlideInTimeImmutable())
        self()->mSlideInTime = v;
    }

    /**
      Get SlideInTime
    */
    static
    int slideInTime()
    {
      return self()->mSlideInTime;
    }

    /**
      Is SlideInTime Immutable
    */
    static
    bool isSlideInTimeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "SlideInTime" ) );
    }

    /**
      Set SlideOutTime
    */
    static
    void setSlideOutTime( int v )
    {
      if (!self()->isSlideOutTimeImmutable())
        self()->mSlideOutTime = v;
    }

    /**
      Get SlideOutTime
    */
    static
    int slideOutTime()
    {
      return self()->mSlideOutTime;
    }

    /**
      Is SlideOutTime Immutable
    */
    static
    bool isSlideOutTimeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "SlideOutTime" ) );
    }

  protected:
    SlidingPopupsConfig(KSharedConfig::Ptr config);
    friend class SlidingPopupsConfigHelper;


    // Effect-slidingpopups
    int mSlideInTime;
    int mSlideOutTime;
};

}

#endif

