// This file is generated by kconfig_compiler_kf6 from colorblindnesscorrectionconfig.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_COLORBLINDNESSCORRECTIONSETTINGS_H
#define KWIN_COLORBLINDNESSCORRECTIONSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {

class ColorBlindnessCorrectionSettings : public KConfigSkeleton
{
  public:

    static ColorBlindnessCorrectionSettings *self();
    static void instance(const QString& cfgfilename);
    static void instance(KSharedConfig::Ptr config);
    ~ColorBlindnessCorrectionSettings() override;

    /**
      Get Mode
    */
    static
    uint mode()
    {
      return self()->mMode;
    }

    /**
      Is Mode Immutable
    */
    static
    bool isModeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Mode" ) );
    }

    /**
      Get Intensity
    */
    static
    double intensity()
    {
      return self()->mIntensity;
    }

    /**
      Is Intensity Immutable
    */
    static
    bool isIntensityImmutable()
    {
      return self()->isImmutable( QStringLiteral( "Intensity" ) );
    }

  protected:
    ColorBlindnessCorrectionSettings(KSharedConfig::Ptr config);
    friend class ColorBlindnessCorrectionSettingsHelper;


    // Effect-colorblindnesscorrection
    uint mMode;
    double mIntensity;
};

}

#endif

