// This file is generated by kconfig_compiler_kf6 from kwinxwaylandsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef XWAYLANDSETTINGS_H
#define XWAYLANDSETTINGS_H

#include <qglobal.h>
#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class XwaylandSettings : public KConfigSkeleton
{
  public:
    class EnumXwaylandEavesdrops
    {
      public:
      enum type { None, Modifiers, Combinations, All, COUNT };
    };

    XwaylandSettings( );
    ~XwaylandSettings() override;

    /**
      Set XwaylandEavesdrops
    */
    void setXwaylandEavesdrops( int v )
    {
      if (!isXwaylandEavesdropsImmutable())
        mXwaylandEavesdrops = v;
    }

    /**
      Get XwaylandEavesdrops
    */
    int xwaylandEavesdrops() const
    {
      return mXwaylandEavesdrops;
    }

    /**
      Is XwaylandEavesdrops Immutable
    */
    bool isXwaylandEavesdropsImmutable() const
    {
      return isImmutable( QStringLiteral( "xwaylandEavesdrops" ) );
    }

    /**
      Set XwaylandEavesdropsMouse
    */
    void setXwaylandEavesdropsMouse( bool v )
    {
      if (!isXwaylandEavesdropsMouseImmutable())
        mXwaylandEavesdropsMouse = v;
    }

    /**
      Get XwaylandEavesdropsMouse
    */
    bool xwaylandEavesdropsMouse() const
    {
      return mXwaylandEavesdropsMouse;
    }

    /**
      Is XwaylandEavesdropsMouse Immutable
    */
    bool isXwaylandEavesdropsMouseImmutable() const
    {
      return isImmutable( QStringLiteral( "xwaylandEavesdropsMouse" ) );
    }

    /**
      Set XwaylandEisNoPrompt
    */
    void setXwaylandEisNoPrompt( bool v )
    {
      if (!isXwaylandEisNoPromptImmutable())
        mXwaylandEisNoPrompt = v;
    }

    /**
      Get XwaylandEisNoPrompt
    */
    bool xwaylandEisNoPrompt() const
    {
      return mXwaylandEisNoPrompt;
    }

    /**
      Is XwaylandEisNoPrompt Immutable
    */
    bool isXwaylandEisNoPromptImmutable() const
    {
      return isImmutable( QStringLiteral( "XwaylandEisNoPrompt" ) );
    }

    /**
      Set XwaylandEisNoPromptApps
    */
    void setXwaylandEisNoPromptApps( const QStringList & v )
    {
      if (!isXwaylandEisNoPromptAppsImmutable())
        mXwaylandEisNoPromptApps = v;
    }

    /**
      Get XwaylandEisNoPromptApps
    */
    QStringList xwaylandEisNoPromptApps() const
    {
      return mXwaylandEisNoPromptApps;
    }

    /**
      Is XwaylandEisNoPromptApps Immutable
    */
    bool isXwaylandEisNoPromptAppsImmutable() const
    {
      return isImmutable( QStringLiteral( "XwaylandEisNoPromptApps" ) );
    }

  protected:

    // Xwayland
    int mXwaylandEavesdrops;
    bool mXwaylandEavesdropsMouse;
    bool mXwaylandEisNoPrompt;
    QStringList mXwaylandEisNoPromptApps;
};

#endif

