/*
    KWin - the KDE window manager
    This file is part of the KDE project.

    SPDX-FileCopyrightText: 2024-2026 Xaver Hugl <xaver.hugl@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/
#include "vulkan_logging.h"
Q_LOGGING_CATEGORY(KWIN_VULKAN, "kwin_vulkan", QtWarningMsg)
