/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.KWin.Plugins.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef PLUGINSADAPTOR_H
#define PLUGINSADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "dbusinterface.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface org.kde.KWin.Plugins
 */
class PluginsAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.KWin.Plugins")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"org.kde.KWin.Plugins\">\n"
"    <property access=\"read\" type=\"as\" name=\"LoadedPlugins\"/>\n"
"    <property access=\"read\" type=\"as\" name=\"AvailablePlugins\"/>\n"
"    <method name=\"LoadPlugin\">\n"
"      <arg direction=\"out\" type=\"b\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"    <method name=\"UnloadPlugin\">\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    PluginsAdaptor(KWin::PluginManagerDBusInterface *parent);
    ~PluginsAdaptor() override;

    inline KWin::PluginManagerDBusInterface *parent() const
    { return static_cast<KWin::PluginManagerDBusInterface *>(QObject::parent()); }

public: // PROPERTIES
    Q_PROPERTY(QStringList AvailablePlugins READ availablePlugins)
    QStringList availablePlugins() const;

    Q_PROPERTY(QStringList LoadedPlugins READ loadedPlugins)
    QStringList loadedPlugins() const;

public Q_SLOTS: // METHODS
    bool LoadPlugin(const QString &name);
    void UnloadPlugin(const QString &name);
Q_SIGNALS: // SIGNALS
};

#endif
