// This file is generated by kconfig_compiler_kf6 from hidecursorconfig.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_HIDECURSORCONFIG_H
#define KWIN_HIDECURSORCONFIG_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {

class HideCursorConfig : public KConfigSkeleton
{
  public:

    static HideCursorConfig *self();
    static void instance(const QString& cfgfilename);
    static void instance(KSharedConfig::Ptr config);
    ~HideCursorConfig() override;

    /**
      Get InactivityDuration
    */
    static
    uint inactivityDuration()
    {
      return self()->mInactivityDuration;
    }

    /**
      Is InactivityDuration Immutable
    */
    static
    bool isInactivityDurationImmutable()
    {
      return self()->isImmutable( QStringLiteral( "InactivityDuration" ) );
    }

    /**
      Get HideOnTyping
    */
    static
    bool hideOnTyping()
    {
      return self()->mHideOnTyping;
    }

    /**
      Is HideOnTyping Immutable
    */
    static
    bool isHideOnTypingImmutable()
    {
      return self()->isImmutable( QStringLiteral( "HideOnTyping" ) );
    }

  protected:
    HideCursorConfig(KSharedConfig::Ptr config);
    friend class HideCursorConfigHelper;


    // Effect-hidecursor
    uint mInactivityDuration;
    bool mHideOnTyping;
};

}

#endif

