// This file is generated by kconfig_compiler_kf6 from nightlightstate.kcfg.
// All changes you do to this file will be lost.
#ifndef NIGHTLIGHTSTATE_H
#define NIGHTLIGHTSTATE_H

#include <qglobal.h>
#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

class NightLightState : public KConfigSkeleton
{
  public:

    NightLightState( );
    ~NightLightState() override;

    /**
      Set State
    */
    void setState( const QString & v )
    {
      if (!isStateImmutable())
        mState = v;
    }

    /**
      Get State
    */
    QString state() const
    {
      return mState;
    }

    /**
      Is State Immutable
    */
    bool isStateImmutable() const
    {
      return isImmutable( QStringLiteral( "State" ) );
    }

  protected:

    // NightLight
    QString mState;
};

#endif

