// This file is generated by kconfig_compiler_kf6 from rulebooksettingsbase.kcfg.
// All changes you do to this file will be lost.

#include "rulebooksettingsbase.h"

using namespace KWin;

RuleBookSettingsBase::RuleBookSettingsBase( KSharedConfig::Ptr config, QObject *parent )
  : KConfigSkeleton( std::move( config ) )
{
  setParent(parent);
  setCurrentGroup( QStringLiteral( "General" ) );

  KConfigSkeleton::ItemInt  *itemCount;
  itemCount = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "count" ), mCount, 0 );
  addItem( itemCount, QStringLiteral( "count" ) );
  QStringList defaultruleGroupList;
  defaultruleGroupList.append( QString::fromUtf8( "QStringList()" ) );

  KConfigSkeleton::ItemStringList  *itemRuleGroupList;
  itemRuleGroupList = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "rules" ), mRuleGroupList, defaultruleGroupList );
  addItem( itemRuleGroupList, QStringLiteral( "ruleGroupList" ) );
}

RuleBookSettingsBase::~RuleBookSettingsBase()
{
}

