// This file is generated by kconfig_compiler_kf6 from kwinxwaylandsettings.kcfg.
// All changes you do to this file will be lost.

#include "kwinxwaylandsettings.h"

KWinXwaylandSettings::KWinXwaylandSettings( QObject *parent )
  : KConfigSkeleton( QStringLiteral( "kwinrc" ) )
{
  setParent(parent);
  KConfigCompilerSignallingItem::NotifyFunction notifyFunction = static_cast<KConfigCompilerSignallingItem::NotifyFunction>(&KWinXwaylandSettings::itemChanged);

  setCurrentGroup( QStringLiteral( "Xwayland" ) );

  QList<KConfigSkeleton::ItemEnum::Choice> valuesxwaylandEavesdrops;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("None");
    valuesxwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Modifiers");
    valuesxwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Combinations");
    valuesxwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("All");
    valuesxwaylandEavesdrops.append( choice );
  }
  KConfigSkeleton::ItemEnum  *innerItemXwaylandEavesdrops;
  innerItemXwaylandEavesdrops = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "XwaylandEavesdrops" ), mXwaylandEavesdrops, valuesxwaylandEavesdrops, Combinations );
  mXwaylandEavesdropsItem = new KConfigCompilerSignallingItem(innerItemXwaylandEavesdrops, this, notifyFunction, signalXwaylandEavesdropsChanged);
  mXwaylandEavesdropsItem->setWriteFlags(KConfigBase::Notify);
  addItem( mXwaylandEavesdropsItem, QStringLiteral( "xwaylandEavesdrops" ) );
  KConfigSkeleton::ItemBool  *innerItemXwaylandEavesdropsMouse;
  innerItemXwaylandEavesdropsMouse = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "XwaylandEavesdropsMouse" ), mXwaylandEavesdropsMouse, false );
  mXwaylandEavesdropsMouseItem = new KConfigCompilerSignallingItem(innerItemXwaylandEavesdropsMouse, this, notifyFunction, signalXwaylandEavesdropsMouseChanged);
  mXwaylandEavesdropsMouseItem->setWriteFlags(KConfigBase::Notify);
  addItem( mXwaylandEavesdropsMouseItem, QStringLiteral( "xwaylandEavesdropsMouse" ) );
  KConfigSkeleton::ItemBool  *innerItemXwaylandEisNoPrompt;
  innerItemXwaylandEisNoPrompt = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "XwaylandEisNoPrompt" ), mXwaylandEisNoPrompt, false );
  mXwaylandEisNoPromptItem = new KConfigCompilerSignallingItem(innerItemXwaylandEisNoPrompt, this, notifyFunction, signalXwaylandEisNoPromptChanged);
  mXwaylandEisNoPromptItem->setWriteFlags(KConfigBase::Notify);
  addItem( mXwaylandEisNoPromptItem, QStringLiteral( "XwaylandEisNoPrompt" ) );
  KConfigSkeleton::ItemStringList  *innerItemXwaylandEisNoPromptApps;
  innerItemXwaylandEisNoPromptApps = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "XwaylandEisNoPromptApps" ), mXwaylandEisNoPromptApps );
  mXwaylandEisNoPromptAppsItem = new KConfigCompilerSignallingItem(innerItemXwaylandEisNoPromptApps, this, notifyFunction, signalXwaylandEisNoPromptAppsChanged);
  mXwaylandEisNoPromptAppsItem->setWriteFlags(KConfigBase::Notify);
  addItem( mXwaylandEisNoPromptAppsItem, QStringLiteral( "XwaylandEisNoPromptApps" ) );
}

int KWinXwaylandSettings::defaultXwaylandEavesdropsValue_helper() const
{

  return Combinations;
}

bool KWinXwaylandSettings::defaultXwaylandEavesdropsMouseValue_helper() const
{

  return false;
}

bool KWinXwaylandSettings::defaultXwaylandEisNoPromptValue_helper() const
{

  return false;
}

KWinXwaylandSettings::~KWinXwaylandSettings()
{
}


void KWinXwaylandSettings::itemChanged(quint64 signalFlag) {

  switch (signalFlag) {
  case signalXwaylandEavesdropsChanged:
    Q_EMIT xwaylandEavesdropsChanged();
    break;
  case signalXwaylandEavesdropsMouseChanged:
    Q_EMIT xwaylandEavesdropsMouseChanged();
    break;
  case signalXwaylandEisNoPromptChanged:
    Q_EMIT XwaylandEisNoPromptChanged();
    break;
  case signalXwaylandEisNoPromptAppsChanged:
    Q_EMIT XwaylandEisNoPromptAppsChanged();
    break;
  }
}
