/****************************************************************************
** Meta object code from reading C++ file 'subsurfacemonitor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kwin-6.7.1/src/utils/subsurfacemonitor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'subsurfacemonitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4KWin17SubSurfaceMonitorE_t {};
} // unnamed namespace

template <> constexpr inline auto KWin::SubSurfaceMonitor::qt_create_metaobjectdata<qt_meta_tag_ZN4KWin17SubSurfaceMonitorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KWin::SubSurfaceMonitor",
        "subSurfaceAdded",
        "",
        "subSurfaceRemoved",
        "subSurfaceMoved",
        "subSurfaceResized",
        "subSurfaceMapped",
        "subSurfaceUnmapped",
        "subSurfaceCommitted",
        "SubSurfaceInterface*",
        "subSurface"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'subSurfaceAdded'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subSurfaceRemoved'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subSurfaceMoved'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subSurfaceResized'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subSurfaceMapped'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subSurfaceUnmapped'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subSurfaceCommitted'
        QtMocHelpers::SignalData<void(SubSurfaceInterface *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SubSurfaceMonitor, qt_meta_tag_ZN4KWin17SubSurfaceMonitorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KWin::SubSurfaceMonitor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin17SubSurfaceMonitorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin17SubSurfaceMonitorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4KWin17SubSurfaceMonitorE_t>.metaTypes,
    nullptr
} };

void KWin::SubSurfaceMonitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SubSurfaceMonitor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->subSurfaceAdded(); break;
        case 1: _t->subSurfaceRemoved(); break;
        case 2: _t->subSurfaceMoved(); break;
        case 3: _t->subSurfaceResized(); break;
        case 4: _t->subSurfaceMapped(); break;
        case 5: _t->subSurfaceUnmapped(); break;
        case 6: _t->subSurfaceCommitted((*reinterpret_cast<std::add_pointer_t<SubSurfaceInterface*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SubSurfaceMonitor::*)()>(_a, &SubSurfaceMonitor::subSurfaceAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubSurfaceMonitor::*)()>(_a, &SubSurfaceMonitor::subSurfaceRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubSurfaceMonitor::*)()>(_a, &SubSurfaceMonitor::subSurfaceMoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubSurfaceMonitor::*)()>(_a, &SubSurfaceMonitor::subSurfaceResized, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubSurfaceMonitor::*)()>(_a, &SubSurfaceMonitor::subSurfaceMapped, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubSurfaceMonitor::*)()>(_a, &SubSurfaceMonitor::subSurfaceUnmapped, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SubSurfaceMonitor::*)(SubSurfaceInterface * )>(_a, &SubSurfaceMonitor::subSurfaceCommitted, 6))
            return;
    }
}

const QMetaObject *KWin::SubSurfaceMonitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KWin::SubSurfaceMonitor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin17SubSurfaceMonitorE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KWin::SubSurfaceMonitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KWin::SubSurfaceMonitor::subSurfaceAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KWin::SubSurfaceMonitor::subSurfaceRemoved()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KWin::SubSurfaceMonitor::subSurfaceMoved()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KWin::SubSurfaceMonitor::subSurfaceResized()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void KWin::SubSurfaceMonitor::subSurfaceMapped()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void KWin::SubSurfaceMonitor::subSurfaceUnmapped()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void KWin::SubSurfaceMonitor::subSurfaceCommitted(SubSurfaceInterface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
