/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.kde.KWin.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "kwinadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class KWinAdaptor
 */

KWinAdaptor::KWinAdaptor(KWin::DBusInterface *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

KWinAdaptor::~KWinAdaptor()
{
    // destructor
}

bool KWinAdaptor::showingDesktop() const
{
    // get the value of property showingDesktop
    return qvariant_cast< bool >(parent()->property("showingDesktop"));
}

QString KWinAdaptor::activeOutputName()
{
    // handle method call org.kde.KWin.activeOutputName
    return parent()->activeOutputName();
}

int KWinAdaptor::currentDesktop()
{
    // handle method call org.kde.KWin.currentDesktop
    return parent()->currentDesktop();
}

QVariantMap KWinAdaptor::getWindowInfo(const QString &in0)
{
    // handle method call org.kde.KWin.getWindowInfo
    return parent()->getWindowInfo(in0);
}

void KWinAdaptor::killWindow()
{
    // handle method call org.kde.KWin.killWindow
    parent()->killWindow();
}

void KWinAdaptor::nextDesktop()
{
    // handle method call org.kde.KWin.nextDesktop
    parent()->nextDesktop();
}

void KWinAdaptor::previousDesktop()
{
    // handle method call org.kde.KWin.previousDesktop
    parent()->previousDesktop();
}

QVariantMap KWinAdaptor::queryWindowInfo()
{
    // handle method call org.kde.KWin.queryWindowInfo
    return parent()->queryWindowInfo();
}

void KWinAdaptor::reconfigure()
{
    // handle method call org.kde.KWin.reconfigure
    parent()->reconfigure();
}

void KWinAdaptor::replace()
{
    // handle method call org.kde.KWin.replace
    parent()->replace();
}

bool KWinAdaptor::setCurrentDesktop(int desktop)
{
    // handle method call org.kde.KWin.setCurrentDesktop
    return parent()->setCurrentDesktop(desktop);
}

void KWinAdaptor::showDebugConsole()
{
    // handle method call org.kde.KWin.showDebugConsole
    parent()->showDebugConsole();
}

void KWinAdaptor::showDesktop(bool showing)
{
    // handle method call org.kde.KWin.showDesktop
    parent()->showDesktop(showing);
}

QString KWinAdaptor::supportInformation()
{
    // handle method call org.kde.KWin.supportInformation
    return parent()->supportInformation();
}


#include "moc_kwinadaptor.cpp"
