/*
    SPDX-FileCopyrightText: 2026 Vlad Zahorodnii <vlad.zahorodnii@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "scene/ninepatch.h"

namespace KWin
{

NinePatch::NinePatch()
{
}

NinePatch::~NinePatch()
{
}

} // namespace KWin
