// This file was generated by kcmutils_generate_module_data(): DO NOT EDIT!

#ifndef KWINXWAYLANDDATA_H
#define KWINXWAYLANDDATA_H

#include <QObject>

#include "kcmoduledata.h"



class KWinXwaylandSettings;


class KWinXwaylandData : public KCModuleData
{
    Q_OBJECT

public:
    explicit KWinXwaylandData(QObject *parent = nullptr);
    KWinXwaylandSettings *settings() const;


private:
    KWinXwaylandSettings *m_settings;

};



#endif
