// This file is generated by kconfig_compiler_kf6 from kwintouchscreenedgeeffectsettings.kcfg.
// All changes you do to this file will be lost.

#include "kwintouchscreenedgeeffectsettings.h"

using namespace KWin;

KWinTouchScreenEdgeEffectSettings::KWinTouchScreenEdgeEffectSettings( const QString & EffectName, QObject *parent )
  : KConfigSkeleton( QStringLiteral( "kwinrc" ) )
  , mParamEffectName(EffectName)
{
  setParent(parent);
  setCurrentGroup( QStringLiteral( "Effect-%1" ).arg( mParamEffectName ) );

  QList<int> defaultTouchBorderActivate;
  defaultTouchBorderActivate.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemTouchBorderActivate;
  itemTouchBorderActivate = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "TouchBorderActivate" ), mTouchBorderActivate, defaultTouchBorderActivate );
  addItem( itemTouchBorderActivate, QStringLiteral( "TouchBorderActivate" ) );
}

QList<int> KWinTouchScreenEdgeEffectSettings::defaultTouchBorderActivateValue_helper() const
{
  QList<int> defaultTouchBorderActivate;
  defaultTouchBorderActivate.append( ElectricNone );

  return defaultTouchBorderActivate;
}

KWinTouchScreenEdgeEffectSettings::~KWinTouchScreenEdgeEffectSettings()
{
}

